<?php

namespace KnitPay\Extensions\AWPCP;

use Pronamic\WordPress\Money\TaxedMoney;
use Pronamic\WordPress\Pay\Payments\Items;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use AWPCP_Payment_Transaction;

/**
 * Title: AWP Classifieds extension
 * Description:
 * Copyright: 2020 Knit Pay
 * Company: Knit Pay
 *
 * @author  knitpay
 * @since   2.5
 */
class PaymentData extends Pay_PaymentData {

	/**
	 * Transaction
	 *
	 * @var AWPCP_Payment_Transaction
	 */
	private $transaction;

	/**
	 * Constructs and initializes an LearnPress payment data object.
	 *
	 * @param   AWPCP_Payment_Transaction $transaction   Instance AWPCP_Payment_Transaction for the order being processed
	 */
	public function __construct( AWPCP_Payment_Transaction $transaction ) {
		$this->transaction = $transaction;
		$this->order_id    = $transaction->get( 'ad-id' );
		$user_id           = $transaction->user_id;

		$this->user_info = (object) array(
			'first_name' => awpcp_post_param( 'first_name', '' ),
			'last_name'  => awpcp_post_param( 'last_name', '' ),
			'user_email' => awpcp_post_param( 'email', '' ),
			'phone'      => awpcp_post_param( 'phone', '' ),
		);
		if ( ! empty( $user_id ) ) {
			$this->save_user_date( $user_id );

			$fields          = array( 'first_name', 'last_name', 'user_email', 'awpcp-profile' );
			$this->user_info = awpcp_users_collection()->find_by_id( $user_id, $fields );
		}

		parent::__construct();
	}

	private function save_user_date( $user_id ) {
		if ( ! ( isset( $_POST['step'] ) && 'checkout' === $_POST['step'] ) ) {
			$profile          = (array) get_user_meta( $user_id, 'awpcp-profile', true );
			$profile['phone'] = $this->user_info->phone;
			update_user_meta( $user_id, 'awpcp-profile', $profile );
			do_action( 'awpcp-user-profile-updated', $profile, $user_id );

			update_user_meta( $user_id, 'first_name', $this->user_info->first_name );
			update_user_meta( $user_id, 'last_name', $this->user_info->last_name );
		}
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'awp-classifieds';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return $this->order_id;
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		/* translators: %s: order id */
		return sprintf( __( 'AWP Classified Ad %s', 'knit-pay' ), $this->order_id );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
		$description = get_awpcp_option( 'knit_pay_payment_description' );

		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{listing_id}' => $this->order_id,
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return $this->transaction->id;
	}

	/**
	 * Get items
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_items()
	 * @return Items
	 */
	/*
	  public function get_items() {

		$payment_lines = new PaymentLines();
		if ( $items = LP()->get_cart()->get_items() ) {
			foreach ( $items as $item ) {

					$line = $payment_lines->new_line();

					$line->set_id( $item['item_id'] );
					$line->set_name( get_the_title( $item['item_id'] ) );
					$line->set_quantity( $item['quantity'] );
					$line->set_discount_amount( new Money($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
					$line->set_unit_price( new TaxedMoney($item['data']->get_origin_price(), $this->get_currency_alphabetic_code()) );
					$line->set_total_amount( new TaxedMoney($item['data']->get_sale_price(), $this->get_currency_alphabetic_code()) );
					$line->set_product_url( get_permalink( $item['item_id'] ) );
				}
			}

			ppp($payment_lines);ddd();
	} */
	public function get_items() {

	}

	public function get_amount() {
		$totals = $this->transaction->get_totals();
		$amount = new TaxedMoney( $totals['money'], $this->get_currency_alphabetic_code() );
		return $amount;
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
		return get_awpcp_option( 'currency-code' );
	}

	/**
	 * Get email.
	 *
	 * @return string
	 */
	public function get_email() {
		if ( isset( $this->user_info->user_email ) ) {
			return $this->user_info->user_email;
		}
	}

	/**
	 * Get first name.
	 *
	 * @return string
	 */
	public function get_first_name() {
		if ( isset( $this->user_info->first_name ) ) {
			return $this->user_info->first_name;
		}
	}

	/**
	 * Get last name.
	 *
	 * @return string
	 */
	public function get_last_name() {
		if ( isset( $this->user_info->last_name ) ) {
			return $this->user_info->last_name;
		}
	}

	/**
	 * Get customer name.
	 *
	 * @return string
	 */
	public function get_customer_name() {
		return $this->get_first_name() . ' ' . $this->get_last_name();
	}

	/**
	 * Get address.
	 *
	 * @return null|string
	 */
	public function get_address() {
		if ( isset( $this->user_info->address ) ) {
			return $this->user_info->address;
		}
	}

	/**
	 * Get city.
	 *
	 * @return null|string
	 */
	public function get_city() {
		if ( isset( $this->user_info->city ) ) {
			return $this->user_info->city;
		}
	}

	/**
	 * Get telephone number.
	 *
	 * @return null|string
	 */
	public function get_telephone_number() {
		if ( isset( $this->user_info->phone ) ) {
			return $this->user_info->phone;
		}
	}
}
